@@ -3,4 +3,7 @@ from .nack_exception import NackException
 from .rpc_provider_exception import RpcProviderException
 from .timeout_exception import TimeoutException
 from .response_timeout_exception import ResponseTimeoutException
-from .publish_timeout_exception import PublishTimeoutException
\ No newline at end of file
+from .publish_timeout_exception import PublishTimeoutException
+from .auto_reconnect_exception import AutoReconnectException
+from .blocking_exception import BlockingException
+from .thread_unsafe_exception import ThreadUnsafeException
