@@ -1,5 +1,7 @@
 from pika.adapters.select_connection import IOLoop
-from time import time, sleep
+from time import time
+
+
 class IOLoopFactory:
     ioloop = IOLoop()
     last_creation = time()
@@ -9,7 +11,7 @@ class IOLoopFactory:
     @classmethod
     def get_ioloop(cls):
         return cls.ioloop
-    
+
     @classmethod
     def reset(cls):
         cls.last_creation = time()
@@ -18,18 +20,17 @@ class IOLoopFactory:
         cls.ioloop = IOLoop()
         cls._reconnect()
         cls.start()
-    
+
     @classmethod
     def start(cls):
         if not cls.running:
-            cls.running=True
+            cls.running = True
             cls.ioloop.start()
 
     @classmethod
     def add_reconnection(cls, call):
         cls._reconnection_list.append(call)
-    
+
     @classmethod
     def _reconnect(cls):
         [reconnect() for reconnect in cls._reconnection_list]
-            
\ No newline at end of file
